@@ -1,5 +1,5 @@
 /*
- * arping 0.3
+ * arping 0.4
  *
  * By marvin@nss.nu
  *
@@ -12,7 +12,7 @@
  *
  * Also finds out IP of specified MAC
  *
- * $Id: arping.c,v 1.8 2000/05/17 23:40:03 marvin Exp $
+ * $Id: arping.c,v 1.11 2000/05/18 08:16:03 marvin Exp $
  */
 /*
  *  Copyright (C) 2000 Marvin (marvin@nss.nu)
@@ -37,7 +37,7 @@
 #include <net/if.h>
 #include <net/if_arp.h>
 
-const float version = 0.3;
+const float version = 0.4;
 
 struct ether_addr *mymac;
 u_char eth_xmas[ETH_ALEN] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
@@ -59,10 +59,16 @@ int verbose = 0;
 int numsent = 0;
 int numrecvd = 0;
 int searchmac = 0;
+int maxcount = 0x7fffffff;
+int rawoutput = 0;
+
+void sigint(int i);
+
 
 void usage(int ret)
 {
-	printf("arping %1.1f [ -v ] [ -i <interface> ] <host/ip/MAC>\n",
+	printf("arping %1.1f [ -v ] [ -r ] [ -c count ] [ -i <interface> ] "
+	       " <host/ip/MAC>\n",
 	       version);
 	exit(ret);
 }
@@ -82,7 +88,7 @@ void alasend(int i)
 				       LIBNET_ICMP_ECHO_H) == -1) { 
 			libnet_error(LIBNET_ERR_FATAL,
 				     "libnet_do_checksum failed\n"); 
-		} 
+		}
 		
 		if (libnet_do_checksum(packet + LIBNET_ETH_H,
 				       IPPROTO_IP, LIBNET_IP_H) == -1) { 
@@ -99,26 +105,32 @@ void alasend(int i)
 		exit(1);
 	}
 	numsent++;
+	if (numsent > maxcount) {
+		sigint(0);
+	}
 	alarm(1);
 }
 
 void sigint(int i)
 {
-	if (searchmac) {
-		u_char *cp=eth_target;
-		int c;
-		printf("\n--- ");
-		for (c = 0; c < ETH_ALEN-1; c++) {
-			printf("%.2x:", (u_char)*cp++);
+	if (!rawoutput) {
+		if (searchmac) {
+			u_char *cp=eth_target;
+			int c;
+			printf("\n--- ");
+			for (c = 0; c < ETH_ALEN-1; c++) {
+				printf("%.2x:", (u_char)*cp++);
+			}
+			printf("%.2x statistics ---\n", *cp);
+		} else {
+			printf("\n--- %s statistics ---\n",
+			       libnet_host_lookup(dip,0));
 		}
-		printf("%.2x statistics ---\n", *cp);
-	} else {
-		printf("\n--- %s statistics ---\n", libnet_host_lookup(dip,0));
+		printf("%d packets transmitted, %d packets recieved, %3.0f%% "
+		       "unanswered\n",
+		       numsent, numrecvd,
+		       100.0 - 100.0 * (float)(numrecvd)/(float)numsent);
 	}
-	printf("%d packets transmitted, %d packets recieved, %3.0f%% "
-	       "unanswered\n",
-	       numsent, numrecvd,
-	       100.0 - 100.0 * (float)(numrecvd)/(float)numsent);
 	exit(1);
 }
 
@@ -143,15 +155,19 @@ void handlepacket(const char *unused, struct pcap_pkthdr *h, u_char *packet)
 			&& !memcmp(eth->h_dest, mymac->ether_addr_octet,
 				   ETH_ALEN)) {
 			u_char *cp = eth->h_source;
-			
-			printf("%d bytes from %s (", h->len,
-			       libnet_host_lookup(hip->saddr, 0));
-			
-			for (c = 0; c < ETH_ALEN-1; c++) {
-				printf("%.2x:", *cp++);
+			if (!rawoutput) {
+				printf("%d bytes from ", h->len);
 			}
-			printf("%.2x): icmp_seq=%d\n", *cp,
-			       hicmp->un.echo.sequence);
+			printf("%s",libnet_host_lookup(hip->saddr, 0));
+			if (!rawoutput) {
+				printf(" (");
+				for (c = 0; c < ETH_ALEN-1; c++) {
+					printf("%.2x:", *cp++);
+				}
+				printf("%.2x): icmp_seq=%d", *cp,
+				       hicmp->un.echo.sequence);
+			}
+			printf("\n");
 		}
 	} else {
 		harp = (struct arphdr*)((char*)eth + sizeof(struct ethhdr));
@@ -163,13 +179,20 @@ void handlepacket(const char *unused, struct pcap_pkthdr *h, u_char *packet)
 					      + harp->ar_hln);
 			if (dip == ip) {
 				cp = (u_char*)harp + sizeof(struct arphdr);
-				printf("%d bytes from ", h->len);
+				if (!rawoutput) {
+					printf("%d bytes from ", h->len);
+				}
 				for (c = 0; c < harp->ar_hln -1; c++) {
 					printf("%.2x:", *cp++);
 				}
-				printf("%.2x (%s): index=%d\n", *cp,
-				       libnet_host_lookup(ip, 0), numrecvd++);
-			}
+				printf("%.2x", *cp);
+				if (!rawoutput) {
+					printf(" (%s): index=%d",
+					       libnet_host_lookup(ip, 0),
+					       numrecvd++);
+				}
+				printf("\n");
+				}
 		}
 	}
 
@@ -190,7 +213,7 @@ int main(int argc, char **argv)
 	int c;
 	struct bpf_program bp;
 	
-	while ((c = getopt(argc, argv, "vhi:")) != EOF) {
+	while ((c = getopt(argc, argv, "vhi:rc:")) != EOF) {
 		switch (c) {
 		case 'v':
 			verbose++;
@@ -200,6 +223,12 @@ int main(int argc, char **argv)
 		case 'i':
 			ifname = optarg;
 			break;
+		case 'r':
+			rawoutput = 1;
+			break;
+		case 'c':
+			maxcount = atoi(optarg);
+			break;
 		default:
 			usage(1);
 		}
@@ -357,10 +386,12 @@ int main(int argc, char **argv)
 	 */
 	signal(SIGALRM, alasend);
 	signal(SIGINT, sigint);
-	if (searchmac) {
-		printf("ARPING %s\n", argv[optind]);
-	} else {
-		printf("ARPING %s\n", libnet_host_lookup(dip,0));
+	if (!rawoutput) {
+		if (searchmac) {
+			printf("ARPING %s\n", argv[optind]);
+		} else {
+			printf("ARPING %s\n", libnet_host_lookup(dip,0));
+		}
 	}
 	alasend(0);
 	for(;;) {
